﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using RTedge; // RT edge system API
using EdgeCode; // RT edge template code

namespace HmiApp
{
    public partial class MainForm : Form
    {
        edgeCode    EgTEMP = null;      // RTedge フレームワーク +API を使用したコードクラス
        object      objValue = null;
        string      strTagSw0;          // タグ名称　SWITCH_0
        string      strTagSw1;          // タグ名称　SWITCH_1
        string      strTagDispBt;       // タグ名称　DISP_BUTTON
        string      strTagLed0;         // タグ名称　LED_0
        string      strTagLed1;         // タグ名称　LED_1
        Color       colSwitchOff;
        Color       colSwitchOn;
        Color       colLEDOff;
        Color       colLEDOn;
        Color       colBtnOff;
        Color       colBtnOn;

        public MainForm()
        {
            InitializeComponent();
            objValue = new object();            // 汎用値オブジェクトの作成
            EgTEMP = new edgeCode();            // RTedge 処理クラスの作成
            EgTEMP.mInstMainClose = Stop;       // フレームワーク内終了メッセージ処理時の関数指定
            EgTEMP.InitService();               // RTedge 処理クラスの初期化

            timUpdate.Interval = (int)EgTEMP.STATE.Cycle;

            try
            {
                // XMLからタグ名称の取得
                if( 0 < EgTEMP.EGAPI.EGFW.tagRef_OUT_List.Count )
                    strTagSw0 = EgTEMP.EGAPI.EGFW.tagRef_OUT_List[0].Name;

                if (1 < EgTEMP.EGAPI.EGFW.tagRef_OUT_List.Count)
                    strTagSw1 = EgTEMP.EGAPI.EGFW.tagRef_OUT_List[1].Name;

                if (2 < EgTEMP.EGAPI.EGFW.tagRef_OUT_List.Count)
                    strTagLed0 = EgTEMP.EGAPI.EGFW.tagRef_OUT_List[2].Name;

                if (3 < EgTEMP.EGAPI.EGFW.tagRef_OUT_List.Count)
                    strTagLed1 = EgTEMP.EGAPI.EGFW.tagRef_OUT_List[3].Name;

                if (0 < EgTEMP.EGAPI.EGFW.tagRef_IN_List.Count)
                    strTagDispBt = EgTEMP.EGAPI.EGFW.tagRef_IN_List[0].Name;

                colSwitchOff = Color.Blue;
                colSwitchOn  = Color.Red;
                colLEDOff    = Color.Gray;
                colLEDOn     = Color.Yellow;
                colBtnOff    = Color.Gray; 
                colBtnOn     = Color.Green;

                rdoSwitch_0.Text = @"OFF";
                rdoSwitch_0.BackColor = colSwitchOff;
                rdoSwitch_0.FlatAppearance.CheckedBackColor = colSwitchOn;
                rdoSwitch_1.Text = @"OFF";
                rdoSwitch_1.BackColor = colSwitchOff;
                rdoSwitch_1.FlatAppearance.CheckedBackColor = colSwitchOn;
                rdoLight_0.Text = @"OFF";
                rdoLight_0.BackColor = colLEDOff;
                rdoLight_0.FlatAppearance.CheckedBackColor = colLEDOn;
                rdoLight_1.Text = @"OFF";
                rdoLight_1.BackColor = colLEDOff;
                rdoLight_1.FlatAppearance.CheckedBackColor = colLEDOn;
                chkDispBtn.Text = @"OFF";
                chkDispBtn.BackColor = colBtnOff;
                chkDispBtn.FlatAppearance.CheckedBackColor = colBtnOn;
            }
            catch
            {
                ;
            }
        }

        /// <summary>
        /// サービス終了処理
        /// </summary>
        public void Stop()
        {
            // アプリケーション終了処理
            if (null != EgTEMP)
            {
                EgTEMP.KillService();
            }
        }

        /// <summary>
        /// フォームロード時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            

            timUpdate.Enabled = true;
        }

        /// <summary>
        /// 画面表示更新処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timUpdate_Tick(object sender, EventArgs e)
        {
            try
            {
                // Tag: SWITCH_0 の値取得
                if (null != strTagSw0)
                {
                    if (0 == EgTEMP.EGAPI.EgTagRead(strTagSw0, ref objValue, false))
                    { // 0: 成功か ?
                        rdoSwitch_0.Checked = (bool)objValue;
                    }
                    else
                    {
                        rdoSwitch_0.Checked = false;
                    }
                }

                // Tag: SWITCH_1 の値取得
                if (null != strTagSw1)
                {
                    if (0 == EgTEMP.EGAPI.EgTagRead(strTagSw1, ref objValue, false))
                    { // 0: 成功か ?
                        rdoSwitch_1.Checked = (bool)objValue;
                    }
                    else
                    {
                        rdoSwitch_1.Checked = false;
                    }
                }

                // Tag: LED_0 の値取得
                if (null != strTagLed0)
                {
                    if (0 == EgTEMP.EGAPI.EgTagRead(strTagLed0, ref objValue, false))
                    { // 0: 成功か ?
                        rdoLight_0.Checked = (bool)objValue;
                    }
                    else
                    {
                        rdoLight_0.Checked = false;
                    }
                }

                // Tag: LED_1 の値取得
                if (null != strTagLed1)
                {
                    if (0 == EgTEMP.EGAPI.EgTagRead(strTagLed1, ref objValue, false))
                    { // 0: 成功か ?
                        rdoLight_1.Checked = (bool)objValue;
                    }
                    else
                    {
                        rdoLight_1.Checked = false;
                    }
                }

                // Tag: DISP_BUTTON の値取得
                if (null != strTagDispBt)
                {
                    objValue = chkDispBtn.Checked;
                    EgTEMP.EGAPI.EgTagWrite(strTagDispBt, objValue);
                }
            }
            catch
            {
                rdoSwitch_0.Checked = false;
                rdoSwitch_1.Checked = false;
                rdoLight_0.Checked = false;
            }


            // ON/OFF別 文字列表示処理
            try
            {
                rdoSwitch_0.Text = (rdoSwitch_0.Checked) ? @"ON" : @"OFF";
                if (!rdoSwitch_0.Checked) { // false時、背景を青に
                    rdoSwitch_0.BackColor = colSwitchOff;
                }

                rdoSwitch_1.Text = (rdoSwitch_1.Checked) ? @"ON" : @"OFF";
                if (!rdoSwitch_1.Checked) { // false時、背景を青に
                    rdoSwitch_1.BackColor = colSwitchOff;
                }

                rdoLight_0.Text = (rdoLight_0.Checked) ? @"ON" : @"OFF";
                if (!rdoLight_0.Checked) { // false時、背景を青に
                    rdoLight_0.BackColor = colLEDOff;
                }

                rdoLight_1.Text = (rdoLight_1.Checked) ? @"ON" : @"OFF";
                if (!rdoLight_1.Checked) { // false時、背景を青に
                    rdoLight_1.BackColor = colLEDOff;
                }

                EgTEMP.UpdateIndicator();
            }
            catch
            {
                rdoSwitch_0.Text = @"OFF";
                rdoSwitch_0.BackColor = colSwitchOff;
                rdoSwitch_1.Text = @"OFF";
                rdoSwitch_1.BackColor = colSwitchOff;
                rdoLight_0.Text = @"OFF";
                rdoLight_0.BackColor = colLEDOff;
                rdoLight_1.Text = @"OFF";
                rdoLight_1.BackColor = colLEDOff;
            }

            
        }

        /// <summary>
        /// 終了ボタン処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdExit_Click(object sender, EventArgs e)
        {
            
            if( DialogResult.Cancel != MessageBox.Show(
                @"RT-edgeサンプルシステムを終了します。よろしいですか？", 
                @"RT-edgeサンプルシステム", 
                MessageBoxButtons.OKCancel, 
                MessageBoxIcon.Information, 
                MessageBoxDefaultButton.Button2) 
                )
            {
                cmdExit.Enabled = false;

                // RT-edgeシステム終了ランチャーの起動
                System.Diagnostics.Process.Start(@"EgShDown.exe");
            }
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            Stop();
        }
        
        private void chkDispBtn_CheckedChanged(object sender, EventArgs e)
        {
            chkDispBtn.Text = (chkDispBtn.Checked) ? @"ON" : @"OFF";
            chkDispBtn.BackColor = (chkDispBtn.Checked) ? colBtnOn: colBtnOff;    // 背景をtrue時On色、false時OFF色に
        }

        private void rdoLight_1_CheckedChanged(object sender, EventArgs e) {

        }
    }
}
