﻿using System;
using System.Threading.Tasks;
using RTedge;   // RT-edge system API
using System.Runtime.CompilerServices;
using System.Threading;
using System.Collections.Generic;

namespace EdgeCode
{
    /// <summary>
    /// RT-edge API/フレームワーク 各種値定義クラス
    /// </summary>
    public static class EDGECONST
    {
        /// <summary>
        /// 各種定義追加
        /// </summary>

        ////
        /// サービス名
        //
        public const string SERVICENAME = "HmiApp";                                             // (*******必須*******) : exe拡張子を除くプログラムファイル名に変更

        ////
        /// サービスインジケータ名
        //        
        public const string IND_MYSERV_STATUS = "SERVICE." + SERVICENAME + ".Status";           // サービス起動
        public const bool SERVICE_UP = true;                                                    // 起動中
        public const bool SERVICE_DOWN = false;                                                 // 停止中
        public const string IND_MYSERV_ERROR = "SERVICE." + SERVICENAME + ".Error";             // サービスエラー発生中
        public const string IND_MYSERV_RUN = "SERVICE." + SERVICENAME + ".Run";		            // サービス実行中
        public const string IND_MYSERV_LIVE = "SERVICE." + SERVICENAME + ".Live";               // サービス実行カウンタ
        
        ////
        /// サービスプロパティタグ名
        //        
        public const string PROP_MYSERV_CYCLE = "SERVICE." + SERVICENAME + ".Cycle";            // サービス動作サイクル(ms)
        public const int DEFAULT_CYCLE = 100;
    }

    /// <summary>
    /// タグ名クラス
    /// </summary>
    public class EgHgObject
    {
        public string Name;     // 名前
    }

    /// <summary>
    /// このコンテナで保持するTagの参照情報
    /// </summary>
    public class EgHgTagRef : EgHgObject
    {
        public ushort Size;     // データサイズ
        public ushort Type;     // データタイプ
        public string Address;  // アドレス情報    
    }

    /// <summary>
    /// RT-edge API/フレームワーク実装クラス
    /// </summary>
    public class edgeCode
    {
        ////
        /// サービス内部状態
        //
        public class MYSTATE
        {
            public bool State;                  // インジケータ.エッジシステム接続状態
            public bool Run;                    // インジケータ.サービス実行状態
            public bool Error;                  // インジケータ.サービス異常状態
            public UInt32 Live;                 // インジケータ.サービス実行カウンタ
            public UInt64 StartTime;            // インジケータ.サービス開始時のタイムスタンプ
            public UInt32 Cycle;                // プロパティ.サービス実行サイクル(ms)	デフォルト100ms


            // 任意ステータス追加可
            public MYSTATE()
            {
                State = false;
                Run = false;
                Error = false;
                Live = 0;
                StartTime = 0;
                Cycle = EDGECONST.DEFAULT_CYCLE;
            }
        }

        ////
        /// edge_APIクラス インスタンス化 
        //
        public edge_API         EGAPI = null;               // edge APIのインスタンス
        public MYSTATE          STATE = null;               // サービス状態クラスのインスタンス
        public List<EgHgTagRef> TAGREF_OUTS;                // RTCDからOutするEgTagRefのリスト
        
        ////
        /// メイン処理終了用コールバック関数
        //
        public delegate void CallbackMainClose();
        public CallbackMainClose mInstMainClose = null;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="myMsgHndlers">メッセージハンドラー関数を指定</param>
        public edgeCode()
        {
            EGAPI = new edge_API();
            STATE = new MYSTATE();
            TAGREF_OUTS = new List<EgHgTagRef>();
        }

        /// <summary>
        /// Edgeメッセージハンドラ
        /// [TODO] : 必要に応じてメッセージ・処理を追加
        /// </summary>
        /// <param name="senderName">メッセージ送信元</param>
        /// <param name="messNo">メッセージ番号</param>
        /// <param name="paramArray">メッセージパラメータ</param>
        /// <returns></returns>
        object WMLOCK = new object();
        public Int32 MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                int result = 0;
                lock (WMLOCK) //排他制御 頻繁にやり取りする場合用 不要の場合は削除
                {
                    switch (messNo) //メッセージ番号毎に処理
                    {
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:    //サービス開始指令
                            result = StartService();
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:   // サービス終了指令
                            {
                                if (mInstMainClose != null)
                                    mInstMainClose();               // コールバック関数からは戻りません
                                else
                                    result = KillService();
                            }
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE:  // サービス一時停止指令
                            result = PauseService();
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_UPDATE: // サービスアップデート指令
                            break;

                        //
                        // [TODO] : 任意のメッセージ処理を追加
                        //
                        default:
                            break;
                    }
                }
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// サービスインジケータの更新 ※更新に失敗しても無視して継続します
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        public int UpdateIndicator()
        {
            try
            {
                // インジケータタグはフレームワークによって自動生成済
                int result = 0;
                ++STATE.Live;
                result |= EGAPI.EgTagWrite(EDGECONST.IND_MYSERV_STATUS, STATE.State);   // サービス起動状態
                result |= EGAPI.EgTagWrite(EDGECONST.IND_MYSERV_RUN, STATE.Run);        // サービス実行状態
                result |= EGAPI.EgTagWrite(EDGECONST.IND_MYSERV_LIVE, STATE.Live);      // サービス実行カウンタ
                result |= EGAPI.EgTagWrite(EDGECONST.IND_MYSERV_ERROR, STATE.Error);    // サービス異常状態
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// サービスプロパティの取得 ※取得できない時はデフォルト設定値を採用します
        /// [TODO] : 必要に応じて設定項目の増減指定
        /// </summary>
        /// <returns></returns>
        public int GetProperty()
        {
            try
            {
                // プロパティタグはXML定義可能　無いときはデフォルト設定値を採用します
                int result = 0;
                object value = new object();

                result = EGAPI.EgTagRead(EDGECONST.PROP_MYSERV_CYCLE, ref value);   // サービス処理サイクルの取得
                if (result == 0)
                    STATE.Cycle = (uint)value;
                else
                    STATE.Cycle = EDGECONST.DEFAULT_CYCLE;                          // デフォルト

                          
                
                //
                // [TODO] : 任意のサービスプロパティを指定可
                //

                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// サービス開始 ※入出力可能になります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        public int StartService()
        {
            try
            {
                STATE.Run = true;       // サービス実行
                UpdateIndicator();      // サービスインジケータ更新
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// サービス一時停止 ※入出力されなくなります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns></returns>
        public int PauseService()
        {
            try
            {
                STATE.Run = false;      // サービス一時停止
                UpdateIndicator();      // サービスインジケータ更新
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// サービスの初期化
        /// </summary>
        /// <returns>正常時０</returns>
        public Int32 InitService()
        {
            try
            {
                Int32 result = 0;
                LOG("[LOG] Start " + EDGECONST.SERVICENAME + " Service");
                edge_API.EDGECONFIG config = new edge_API.EDGECONFIG(); //設定情報の初期化
                config = EGAPI.EDGE_CONFIG_DEFAULT;                     //初期値設定

                config.mInstMessFunc = MsgHandler;                      //Edgeメッセージユーザハンドラの指定

                result = EGAPI.EgInit(config);                          // Edgeフレームワークの初期化
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                result = GetProperty();                                 // サービスプロパティのロード処理
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                STATE.State = EDGECONST.SERVICE_UP;                     // サービスアップ

                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// サービス削除（プロセス停止）
        /// </summary>
        /// <returns></returns>
        public int KillService()
        {
            try
            {
                PauseService();                         // サービス一時停止

                STATE.State = EDGECONST.SERVICE_DOWN;   //
                UpdateIndicator();				        // サービスインジケータ更新
                LOG("[LOG] End " + EDGECONST.SERVICENAME + " Service");
                EGAPI.EgFinalize();
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        /// <param name="filePath"></param>
        /// <param name="lineNumber"></param>
        public void WARN(int reason,
                         [CallerFilePath] string filePath = "",
                         [CallerLineNumber] int lineNumber = 0)
        {
            try
            {
                Console.WriteLine(EDGECONST.SERVICENAME + " service warning at file(" + filePath + ") ,line(" + lineNumber.ToString() +
                                   ")  ,reason 0x" + reason.ToString("x8"));
            }
            catch
            { }
        }

        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        public void LOG(string message)
        {
            try
            {
                Console.WriteLine(EDGECONST.SERVICENAME + " : " + message);
            }
            catch
            { }
        }
    }
}
